using System;
using System.IO;
using System.Text;
using System.Reflection;
using System.Collections.Generic;
using System.Data;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Threading;

using PRoCon.Core;
using PRoCon.Core.Plugin;
using PRoCon.Core.Plugin.Commands;
using PRoCon.Core.Players;
using PRoCon.Core.Players.Items;
using PRoCon.Core.Battlemap;
using PRoCon.Core.Maps;

namespace PRoConEvents
{
    public class CEmptyMap : PRoConPluginAPI, IPRoConPluginInterface
    {

        private string m_strHostName;
        private string m_strPort;
        private string m_strPRoConVersion;

        private int m_iCurrentMap;
        private int m_iEmptyMap;
        private int m_iSrvInfoCurPlayerCount;
        private int m_iTimeEmpty;
        private int m_iWaitTime;
        private bool m_bWaiting;
		
		private int m_iLowPlayerCount;
		private int m_iLowCountWait;
		private string m_enLowCountMessage;
		private int m_iLowCountSpamInterval;
		private int m_iMessageRepeat;
		private bool m_bLowCountPopulating;
		private int m_iMapAfterEmpty;
		
        enumBoolYesNo m_enLowPlayerCount;
		enumBoolYesNo m_enForceLowCountMap;
		enumBoolYesNo m_enDebug;
		
        private bool m_isPluginEnabled;

        public CEmptyMap()
        {


            this.m_iSrvInfoCurPlayerCount = 0;
            this.m_iCurrentMap = 0;
            this.m_iEmptyMap = 0;
            this.m_iTimeEmpty = 3;
			
            this.m_enDebug = enumBoolYesNo.No;
			
			this.m_enLowPlayerCount = enumBoolYesNo.No;
			this.m_enForceLowCountMap = enumBoolYesNo.Yes;
			this.m_iLowPlayerCount = 4;
			this.m_iLowCountWait = 60;
			this.m_iLowCountSpamInterval = 20;
			this.m_enLowCountMessage = "AUTOMATED MESSAGE:PLAYERS ARE LEAVING, SWITCHING TO [EMPTY MAP]";
			this.m_bLowCountPopulating = false;
			this.m_iMapAfterEmpty = m_iEmptyMap+1;
			
            this.m_bWaiting = false;


            this.m_isPluginEnabled = false;
        }

        public string GetPluginName()
        {
            return "Empty Map";
        }

        public string GetPluginVersion()
        {
            return "0.0.5";
        }

        public string GetPluginAuthor()
        {
            return "Fruity";
        }

        public string GetPluginWebsite()
        {
            return "team-aftershocks.com";
        }

        public string GetPluginDescription()
        {
            return @"<h2>Description</h2>
			<p>Sets the server to the map index specified if it is empty for a user defined time period.</p>

<h2>Settings</h2>
	<h4>Empty Map</h4>
		<blockquote><h4>Empty Map</h4>The index for the map you want to run. 0 = first map in rotation, 1 = second map in rotation, 2 = third map in rotation, etc</blockquote>
		<blockquote><h4>Time empty (mins)</h4>How long the server should be empty (in minutes) before switching to the empty map</blockquote>
	<h4>Low Player Count</h4>
		<blockquote><h4>Keep empty map on low player count</h4>Whether the server should continue playing the empty map if there are less than or equal to X players</blockquote>
		<blockquote><h4>Player count on empty map</h4>How many players are required to move to the map after the empty map in the rotation</blockquote>
		<blockquote><h4>Force Map Change</h4>Whether the server should switch to the empty map immediately on the player count running below the Low Player Count</blockquote>
		<blockquote><h4>Delay before forcing map change</h4>How long in seconds should the server wait before switching to the empty map</blockquote>
		<blockquote><h4>Message during wait</h4>The message used to tell players the server is going to the empty map</blockquote>
		<blockquote><h4>Message interval</h4>How often in seconds should the wait message be sent.</blockquote>
	<h4>Extras</h4>
		<blockquote><h4>Debug</h4>Whether the plugin should print what it is doing to the plugin console. Be warned, it spams it quite a bit.</blockquote>

<h2>Development</h2>
	<h3>Changelog</h3>
	
		<h4>0.0.5 - 29 Dec 2011</h4>
		<ul>
			<li>Fixed forced map change not correctly forcing the map and still forcing a map once the server is populated</li>
		</ul>
	
		<h4>0.0.4 - 28 Dec 2011</h4>
		<ul>
			<li>Added a user defined delay until force switch</li>
			<li>Added a basic message sending system to tell players the map is changing</li>
		</ul>
		<h4>0.0.3 - 28 Dec 2011</h4>
		<ul>
			<li>Added ability to force a switch to the empty map when player count is low</li>
		</ul>
		<h4>0.0.2 - 27 Dec 2011</h4>
		<ul>
			<li>Added a low player count option so the server will continue to play the empty map until there is a sufficient number of players to move off it</li>
			<li>Fixed a few debug messages</li>
		</ul>
		<h4>0.0.1 - 24 Dec 2011 - <b>Initial Features</b></h4>
		<ul>
			<li>Set a specific map index</li>
			<li>Set time the server should be empty</li>
			<li>Print debug messages to console</li>
		</ul>
	<h3>To-do</h3>
		<ul>
			<li>Add an idle kick disabler until X players</li>
			<li>Cut down on debug spam</li>
			<li>Your suggestions ?</li>
		</ul>

";
        }

        public void OnPluginLoaded(string strHostName, string strPort, string strPRoConVersion)
        {
            this.m_strHostName = strHostName;
            this.m_strPort = strPort;
            this.m_strPRoConVersion = strPRoConVersion;

            this.RegisterEvents(this.GetType().Name, "OnServerInfo", "OnMaplistGetMapIndices");
        }

        public void OnPluginEnable()
        {
            this.ExecuteCommand("procon.protected.pluginconsole.write", "^bEmpty Map: ^2Enabled!");
            this.ExecuteCommand("procon.protected.send", "serverInfo");
            this.m_isPluginEnabled = true;
        }

        public void OnPluginDisable()
        {
            this.ExecuteCommand("procon.protected.pluginconsole.write", "^bEmpty Map: ^1Disabled =(");
            this.m_isPluginEnabled = false;
        }

        // Lists only variables you want shown.. for instance enabling one option might hide another option 
        // It's the best I got until I implement a way for plugins to display their own small interfaces.
        public List<CPluginVariable> GetDisplayPluginVariables()
        {
            List<CPluginVariable> lstReturn = new List<CPluginVariable>();

            lstReturn.Add(new CPluginVariable("1 Empty Map|Empty Map", typeof(int), this.m_iEmptyMap));
            lstReturn.Add(new CPluginVariable("1 Empty Map|Time empty (mins)", typeof(int), this.m_iTimeEmpty));
			lstReturn.Add(new CPluginVariable("2 Low Player Count|Keep empty map at low players?", typeof(enumBoolYesNo), this.m_enLowPlayerCount));
			if (this.m_enLowPlayerCount == enumBoolYesNo.Yes)
			{
				lstReturn.Add(new CPluginVariable("2 Low Player Count|Player count on empty map", typeof(int), this.m_iLowPlayerCount));
				lstReturn.Add(new CPluginVariable("2 Low Player Count|Force map change?", typeof(enumBoolYesNo), this.m_enForceLowCountMap));
				lstReturn.Add(new CPluginVariable("2 Low Player Count|Delay before forcing change (secs)", typeof(int), this.m_iLowCountWait));
				lstReturn.Add(new CPluginVariable("2 Low Player Count|Message during wait", typeof(string), this.m_enLowCountMessage));
				lstReturn.Add(new CPluginVariable("2 Low Player Count|Message interval (secs)", typeof(int), this.m_iLowCountSpamInterval));
			}
            lstReturn.Add(new CPluginVariable("3 Extra|Debug", typeof(enumBoolYesNo), this.m_enDebug));
            return lstReturn;
        }

        // Lists all of the plugin variables.
        public List<CPluginVariable> GetPluginVariables()
        {
            List<CPluginVariable> lstReturn = new List<CPluginVariable>();

            lstReturn.Add(new CPluginVariable("Empty Map", typeof(int), this.m_iEmptyMap));
            lstReturn.Add(new CPluginVariable("Time empty (mins)", typeof(int), this.m_iTimeEmpty));
			lstReturn.Add(new CPluginVariable("Keep empty map at low players?", typeof(enumBoolYesNo), this.m_enLowPlayerCount));
			if (this.m_enLowPlayerCount == enumBoolYesNo.Yes)
			{
				lstReturn.Add(new CPluginVariable("Player count on empty map", typeof(int), this.m_iLowPlayerCount));
				lstReturn.Add(new CPluginVariable("Force map change?", typeof(enumBoolYesNo), this.m_enForceLowCountMap));
				if (this.m_enForceLowCountMap == enumBoolYesNo.Yes)
				{
					lstReturn.Add(new CPluginVariable("Delay before forcing change (secs)", typeof(int), this.m_iLowCountWait));
					lstReturn.Add(new CPluginVariable("Message during wait", typeof(string), this.m_enLowCountMessage));
					lstReturn.Add(new CPluginVariable("Message interval (secs)", typeof(int), this.m_iLowCountSpamInterval));
				}
			}
            lstReturn.Add(new CPluginVariable("Debug", typeof(enumBoolYesNo), this.m_enDebug));

            return lstReturn;
        }

        // Allways be suspicious of strValue's actual value.  A command in the console can
        // by the user can put any kind of data it wants in strValue.
        // use type.TryParse
        public void SetPluginVariable(string strVariable, string strValue)
        {
            int iHelper;
            if (strVariable.CompareTo("Empty Map") == 0 && int.TryParse(strValue, out iHelper) == true)
            {
                this.m_iEmptyMap = iHelper;
            }
            else if (strVariable.CompareTo("Time empty (mins)") == 0 && int.TryParse(strValue, out iHelper) == true)
            {
                this.m_iTimeEmpty = iHelper;
            }
            else if (strVariable.CompareTo("Keep empty map at low players?") == 0 && Enum.IsDefined(typeof(enumBoolYesNo), strValue) == true)
            {
                this.m_enLowPlayerCount = (enumBoolYesNo)Enum.Parse(typeof(enumBoolYesNo), strValue);
            }
            else if (strVariable.CompareTo("Player count on empty map") == 0 && int.TryParse(strValue, out iHelper) == true)
            {
                this.m_iLowPlayerCount = iHelper;
            }
            else if (strVariable.CompareTo("Force map change?") == 0 && Enum.IsDefined(typeof(enumBoolYesNo), strValue) == true)
            {
                this.m_enForceLowCountMap = (enumBoolYesNo)Enum.Parse(typeof(enumBoolYesNo), strValue);
            }
			else if (strVariable.CompareTo("Delay before forcing change (secs)") == 0 && int.TryParse(strValue, out iHelper) == true)
            {
                this.m_iLowCountWait = iHelper;
            }
			else if (strVariable.CompareTo("Message during wait") == 0) {
                this.m_enLowCountMessage = strValue;
            }
			else if (strVariable.CompareTo("Message interval (secs)") == 0 && int.TryParse(strValue, out iHelper) == true)
            {
                this.m_iLowCountSpamInterval = iHelper;
            }
			else if (strVariable.CompareTo("Debug") == 0 && Enum.IsDefined(typeof(enumBoolYesNo), strValue) == true)
            {
                this.m_enDebug = (enumBoolYesNo)Enum.Parse(typeof(enumBoolYesNo), strValue);
            }

        }

        private void UnregisterAllCommands()
        {
        }

        private void SetupHelpCommands()
        {
        }

        private void RegisterAllCommands()
        {
        }




        public override void OnServerInfo(CServerInfo csiServerInfo)
        {
            this.m_iSrvInfoCurPlayerCount = csiServerInfo.PlayerCount;

            if (this.m_enDebug == enumBoolYesNo.Yes)
            {
                this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Server info has been recieved and player count updated.");
            }
			if (this.m_bWaiting == false)
			{	
				if (this.m_enDebug == enumBoolYesNo.Yes)
				{
					this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Passing over to PlayerCheck()");
				}
				PlayerCheck();
			}else if (this.m_bWaiting == true){
				this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Server info recieved, but we're still waiting...");
        }
        }

        public override void OnMaplistGetMapIndices(int mapIndex, int nextIndex)
        {
            this.m_iCurrentMap = mapIndex;
        }


        public void PlayerCheck()
        {
            if (this.m_bWaiting == false)
            {
				if (this.m_iSrvInfoCurPlayerCount == 0)
				{
					this.m_bLowCountPopulating = false;
					if (this.m_enDebug == enumBoolYesNo.Yes)
					{
					this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Server is empty");
					this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Sending wait command");
					}
					this.m_bWaiting = true;
					if (this.m_enDebug == enumBoolYesNo.Yes)
					{
					this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Wait starting...");
					}
						Thread delayed = new Thread(new ThreadStart(delegate()
						 {
							 int X = m_iTimeEmpty*60;
							 Thread.Sleep(X * 1000);
							 this.m_bWaiting = false;
							 if (this.m_enDebug == enumBoolYesNo.Yes)
							 {
							 this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Wait over, sending map check");
							 }
							 MapCheck();
						 }));
					delayed.Start();
				}
				else if (this.m_enLowPlayerCount == enumBoolYesNo.Yes && this.m_iSrvInfoCurPlayerCount <= this.m_iLowPlayerCount)
				{
					this.ExecuteCommand("procon.protected.send", "mapList.getMapIndices");
					if (this.m_enDebug == enumBoolYesNo.Yes)
					{
						this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Server is on a low player count, setting the empty map to next map");
					}
					this.m_bLowCountPopulating = true;
					this.ExecuteCommand("procon.protected.send", "mapList.setNextMapIndex", this.m_iEmptyMap.ToString());
					this.ExecuteCommand("procon.protected.send", "mapList.getMapIndices");
					if (this.m_enForceLowCountMap == enumBoolYesNo.Yes)
					{
						if (this.m_enDebug == enumBoolYesNo.Yes)
						{	
							this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Doing checks to see if the map should be forced now.");
						}
						if (this.m_iCurrentMap != this.m_iEmptyMap)
						{
							this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Delaying forced map change");
							// Task command
								this.m_iMessageRepeat = this.m_iLowCountWait / this.m_iLowCountSpamInterval;
							this.ExecuteCommand("procon.protected.tasks.add", "CEmptyMap", "0", this.m_iLowCountSpamInterval.ToString(), this.m_iMessageRepeat.ToString(), "procon.protected.send", "admin.say", this.m_enLowCountMessage, "all");
							// Delay before forcing map change to spam the chat with the warning message
							Thread delayed = new Thread(new ThreadStart(delegate()
								{
									int X = m_iLowCountWait;
									Thread.Sleep(X * 1000);
									if (this.m_enDebug == enumBoolYesNo.Yes)
									{
										this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Delay over, forcing map change now");
									}
									if (this.m_iCurrentMap != this.m_iEmptyMap)
									{
										this.ExecuteCommand("procon.protected.send", "mapList.runNextRound");
										this.ExecuteCommand("procon.protected.send", "mapList.getMapIndices");
									}
								}));
								delayed.Start();
						}
						else
						{
							if (this.m_enDebug == enumBoolYesNo.Yes)
							{	
								this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Server is running the empty map with a few players.");
							}
						}
					}
				}
				else if (this.m_enLowPlayerCount == enumBoolYesNo.Yes && this.m_iSrvInfoCurPlayerCount > this.m_iLowPlayerCount)
				{
					if (this.m_enDebug == enumBoolYesNo.Yes)
					{
						this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Server is above low player count");
					}
					if (this.m_bLowCountPopulating == true)
					{
						if (this.m_enDebug == enumBoolYesNo.Yes)
							{
								this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Next map is the map after the empty map in rotation");
							}
						this.ExecuteCommand("procon.protected.send", "mapList.setNextMapIndex", this.m_iMapAfterEmpty.ToString());
						this.m_bLowCountPopulating = false;
					}
				}
				
				else
				{
					if (this.m_enDebug == enumBoolYesNo.Yes)
					{
					this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Server is not empty..."); 
					}
				}
            }
        }
				
		public void MapCheck()
		{
            this.ExecuteCommand("procon.protected.send", "mapList.getMapIndices");
			if (this.m_enDebug == enumBoolYesNo.Yes)
            {
                this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Checking server is still empty");
            }
			
			if (this.m_iSrvInfoCurPlayerCount == 0)
			{
				if (this.m_enDebug == enumBoolYesNo.Yes)
				{
					this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: MapCheck starting...");
				}
					if (this.m_iCurrentMap != this.m_iEmptyMap)
					{
						if (this.m_enDebug == enumBoolYesNo.Yes)
						{
							this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: The map (" + this.m_iCurrentMap.ToString() + ") is not the empty map (" +
							this.m_iEmptyMap.ToString() + "), switching now");
						}
						this.ExecuteCommand("procon.protected.send", "mapList.setNextMapIndex", this.m_iEmptyMap.ToString());
						this.ExecuteCommand("procon.protected.send", "mapList.runNextRound");
						if (this.m_enDebug == enumBoolYesNo.Yes)
						{
							this.ExecuteCommand("procon.protected.plugnconsole.write", "The map has been switched");
						}
					}
					else if (this.m_iCurrentMap == this.m_iEmptyMap)
					{
						if (this.m_enDebug == enumBoolYesNo.Yes)
						{
							this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Server is running the empty map");
						}
					}
			}
			else
			{ 
				if (this.m_enDebug == enumBoolYesNo.Yes)
				{
					this.ExecuteCommand("procon.protected.pluginconsole.write", "Empty Map: Server is no longer empty");
				}
			}
		}	
	}
}